'''
Tests for lib.py
'''
import lib

from robot.api import TestSuiteBuilder
from testspec import TestSpec

def test_docreference():
    '''
    Test creation of a TestReference object
    '''
    
    tref = lib.DocReference([1, 2, 3, 4])

    assert tref.lvl() == 4
    assert str(tref) == "1.2.3.4"

def test_nextreference():
    '''
    Test creation of the subsequent TestReference object
    '''

    tref = lib.DocReference([1, 2])
    next_tref = tref.next_ref()

    assert next_tref.lvl() == 2
    assert str(next_tref) == "1.3"

    assert tref.lvl() == 2
    assert str(tref) == "1.2"

def test_nextreference_lvl3():
    '''
    Test creation of the subsequent TestReference object
    '''

    tref = lib.DocReference([6, 3, 1])
    next_tref = tref.next_ref()

    assert next_tref.lvl() == 3
    assert str(next_tref) == "6.3.2"

def test_subreference():
    '''
    Test creation of the inner TestReference object
    '''

    tref = lib.DocReference([1, 2])
    next_tref = tref.sub_ref()

    assert next_tref.lvl() == 3
    assert str(next_tref) == "1.2.1"

    assert tref.lvl() == 2
    assert str(tref) == "1.2"    

def test_gen_api_suite():
    '''
    Test generation of a Test Suite (i.e. Robot file) 
    '''

    sec_ref = lib.DocReference.from_str("6.4.5.1.1")
    suite = TestSuiteBuilder().build("../examples/SOL002/VNFConfiguration-API/Configuration.robot")
    spec = TestSpec("empty.docx")
    workspace = "Another-API"
    commit_id = "deadbeef"
    sol = "SOL002"

    lib.gen_api_suite(
        sec_ref, suite, spec, workspace, commit_id, sol
    )

    outfn = "../examples/test.docx"
    spec.save(outfn)

def test_build_test_url():
    '''
    Tests building the URL to a test
    '''
    url = lib.build_test_url("dead", "SOL002", "My-API", "My-File.robot") 
    assert url == "https://forge.etsi.org/rep/nfv/api-tests/raw/dead/SOL002/My-API/My-File.robot"
